/*  jview : Image viewer
 *  Copyright (C) 1998 david Ramboz
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __LDB_UTILS_H__
#define __LDB_UTILS_H__

#include <string.h>
#include <netinet/in.h>
#include <glib.h>
#include "ldb.h"


extern inline  void
ldb_write_uint8( LDB *ldb, LDBCursor *cursor, guint8 n) {
    GUINT8( cursor->pos) = n;
    cursor->pos += 1;
}

extern inline  void
ldb_write_uint16( LDB *ldb, LDBCursor *cursor, guint16 n) {
    GUINT16( cursor->pos) = GUINT16BE( n);
    cursor->pos += 2;
}

extern inline  void
ldb_write_uint32( LDB *ldb, LDBCursor *cursor, guint32 n) {
    GUINT32( cursor->pos) = GUINT32BE( n);
    cursor->pos += 4;
}

extern inline  guint8
ldb_read_uint8( LDB *ldb, LDBCursor *cursor) {
    guint8 n;

    n = GUINT8( cursor->pos);
    cursor->pos += 1;
    return n;
}

extern inline  guint16
ldb_read_uint16( LDB *ldb, LDBCursor *cursor) {
    guint16 n;

    n = GUINT16(cursor->pos);
    cursor->pos += 2;
    return GUINT16BE( n);
}

extern inline  guint32
ldb_read_uint32( LDB *ldb, LDBCursor *cursor) {
    guint32 n;

    n = GUINT32(cursor->pos);
    cursor->pos += 4;
    return GUINT32BE( n);
}

extern inline  void
ldb_write_str( LDB *ldb, LDBCursor *cursor, char *str) {
    guint size;

    size = strlen( str) + 1;
    memcpy( cursor->pos, str, size);
    cursor->pos += size;
}


extern inline  char*
ldb_read_str( LDB *ldb, LDBCursor *cursor) {
    char *str;
    gint size;

    size = strlen( cursor->pos) + 1;
    str = g_malloc( size);
    memcpy( str, cursor->pos, size);
    cursor->pos += size;

    return str;
}

extern inline  guint
ldb_str_size( LDB *ldb, char *str) {
    return strlen( str) + 1;
}

/* string lists cannot contains the empty string ("") which is used to
   mark the end of the list
   */

extern inline  void
ldb_write_str_list( LDB *ldb, LDBCursor *cursor, GList *list) {
    while ( list) {
	if ( *((char*) list->data))
	    ldb_write_str( ldb, cursor, (char*) list->data);
	else
	    g_warning( "ldbutils.h : Skipping empty string");
	list = list->next;
    }
    ldb_write_uint8( ldb, cursor, 0);
}

extern inline  GList*
ldb_read_str_list( LDB *ldb, LDBCursor *cursor) {
    GList *list = NULL;
    char *str;

    while ( (str = ldb_read_str( ldb, cursor)) && *str)
	list = g_list_append( list, str);
    g_free( str);

    return list;
}

extern inline  guint
ldb_str_list_size( LDB *ldb, GList *list) {
    gint size = 1;

    while ( list) {
	size += strlen( (char*) list->data) + 1;
	list = list->next;
    }

    return size;
}

#endif /* __LDB_UTILS_H__ */




